function [r tn T P E] = Gmod(Gm,  Gp,  Gv,  Gn,  Gr,  GR,  Gtheta, Gphi,
  tmax, level, soft, vert, out, fname)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PHYS 210 Term Project Component - Galaxy Merger Generator
%
%  Generate the initial positions for a given galaxy merger
%
%  Input arguments:
%	
%	$$$$$$$$ First colliding galaxy $$$$$$$$
%	Gm		- mass of the bulge
%	Gp		- position vector of the center of the galaxy (i.e. [x y z])
%	Gv		- initial velocity vector of this galaxy
%	Gn		- number of stars in the disk (including stars in the bulge)
%	Gr		- buldg raius 
%	GR		- disk radius
%	Gtheta		- disk angle
%	Gphi		- disk angle
%
%  Output arguments:
%
%	r	- position matrix of the single galaxy system
%	nt	- 1D array of time
%	T	- total kinetic energy
%	V	- total potential energy
%	E	- total energy
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nt = 2^level+1;
dt = tmax/(nt-1);
tn = [0.0:dt:tmax];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Generate disk stars' position data for all three galaxies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[pout vout ns] = addsk(Gm,Gp,Gv,Gn,vert,Gr,GR,Gtheta,Gphi);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Initalize arrays for efficiency. Total number of elements equals the 
%  number of stars in each galaxy + 3 bulges
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = ns+1;

%  Position, Velocity, Acceelration array
r = zeros(N, 3, nt);
%  v a p for the bulge are zero or constant, so exclude it.
v = zeros(ns, 3, nt);
a = zeros(ns, 3, nt-1);
%  Potential array
aP = zeros(ns, nt);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Adding the positions & velocities elements to arrays.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Add all particles' position coordinates at t = 0 to the matrix
r(1,:,1) = Gp;
r(2:N,:,1) = pout;

%  Add all partcles' velocities to matrix v
v(:,:,1) = vout;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Add particles' positions at t = 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Only one galaxy in this system, so no acceleration and potential for this galaxy
r(1,:,2) = r(1,:,1) + dt*Gv;

%  acceleration of the stars
MrG = zeros(ns,3);
MrG(:,1) = Gp(1);
MrG(:,2) = Gp(2);
MrG(:,3) = Gp(3);

a(:,:,1) = Gm./(sqrt(magnitude(r(2:N,:,1),MrG)+soft^2)).^3.*( MrG - r(2:N,:,1) );

%aP(:,1) = -Gm./( magsingle( r(2:N,:,1),MrG )+soft );

r(2:N,:,2) = r(2:N,:,1) + dt*v(:,:,1) + 1/2*dt^2*(a(:,:,1));

v(:,:,2) = v(:,:,1) + dt * a(:,:,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Compute the positions of all particles in the system until t max.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for s = 2:length(tn)-1
%  the galaxy is moving at constant velocity, which is vG
r(1,:,s+1) = r(1,:,s) + dt*Gv;

%  Create matrices for element by element operations
tprG = zeros(ns,3);
tprG(:,1) = r(1,1,s);
tprG(:,2) = r(1,2,s);
tprG(:,3) = r(1,3,s);

a(:,:,s) = Gm./(sqrt(magnitude(r(2:N,:,s),MrG)+soft^2)).^3.*( tprG - r(2:N,:,s) );

%aP(:,s) = -Gm./( magsingle( r(2:N,:,s),MrG )+soft );

r(2:N,:,s+1) = 2*r(2:N,:,s) - r(2:N,:,s-1) + dt^2*( a(:,:,s) );

%  Calculate velocities for all particles at all t
v(:,:,s+1) = v(:,:,s) + dt*a(:,:,s);

endfor

%  Calculate potential at t=nt
%tmG = zeros(ns,3);
%tmG(:,1) = r(1,1,nt);
%tmG(:,2) = r(1,2,nt);
%tmG(:,3) = r(1,3,nt);
%aP(:,nt) = -Gm./(magsingle( r(2:N,:,s),tmG )+soft);

%vtot= 0.5*sum(v(:,1,:).^2 + v(:,2,:).^2 + v(:,3,:).^2);
%T=zeros(1,nt);
%T(1,:) = vtot;
%P = sum(aP);

%  Calculate the total E
%E = T + P;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Output data to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

output = out;
if output
	%  Create mass array for the output file
	m=ones(1,N);
	%  mass of G
	m(1,1)=10;

	%  Create colour array
	rgb=zeros(N,3);
	%  Add bulge colors
	rgb(1,:)=[0 1 1];
	%  Add star colors
	cG=zeros(ns,3);
	cG(:,1)=1;
	cG(:,2)=1;
	cG(:,3)=0;
	rgb(2:N,:)=cG;
	rout = r ./ 45;
	mout = m ./ 5;
	%  function arguments: nbodyout(fname, t, r, m, rgb)
	if fname == 0
		nbodyout('merger', tn, rout, mout, rgb);
	else
		nbodyout(fname, tn, rout, mout, rgb);
	endif
endif


end





function R = magnitude(r1, r2)
%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Calculate the scalar distance squared between two objects 
%  using their position vectors
%
%  Input arguments:
%	r1 - position vector of object 1
%	r2 - position vector of object 2
%
%  Output argument:
%	R - the scalar distance squared between the two
%%%%%%%%%%%%%%%%%%%%%%%%%%

tmp = r2-r1;
R = [tmp(:,1).^2+tmp(:,2).^2+tmp(:,3).^2 tmp(:,1).^2+tmp(:,2).^2+tmp(:,3).^2 tmp(:,1).^2+tmp(:,2).^2+tmp(:,3).^2];
end 





function o = magsingle(in1, in2)
%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Calculate the scalar distance between two objects 
%  using their position vectors
%
%  Input arguments:
%	r1 - position vector of object 1
%	r2 - position vector of object 2
%
%  Output argument:
%	R - the scalar distance between the two
%%%%%%%%%%%%%%%%%%%%%%%%%%

tmp = in2-in1;
o = sqrt(tmp(1).^2+tmp(2).^2+tmp(3).^2);
end 
