function [r tn T P E Gns GSns Sns] = Mmodel(Gm, Gp, Gv, Gn, Gr, GR, Gtheta, Gphi,
  GSm, GSp, GSv, GSn, GSr, GSR, GStheta, GSphi,
  Sm, Sp, Sv, Sn, Sr, SR, Stheta, Sphi,
  tmax, level, soft, out, fname)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PHYS 210 Term Project Component - Galaxy Merger Generator
%
%  Generate the initial positions for a given galaxy merger
%
%  Input arguments:
%	
%	$$$$$$$$ First colliding galaxy $$$$$$$$
%	Gm	- mass of the bulge
%	Gp	- position vector of the center of the galaxy (i.e. [x y z])
%	Gv	- initial velocity vector of this galaxy
%	Gn	- number of stars in the disk (including stars in the bulge)
%	Gr	- buldg raius 
%	GR	- disk radius
%	Gtheta	- disk angle
%	Gphi	- disk angle
%
%	Same for GS-the colliding galaxy with a satellite and S-the satellite galaxy.
%
%  Output arguments:
%
%	r	- position matrix of the single galaxy system
%	tn	- 1D array of time
%	T	- total kinetic energy
%	P	- total potential energy
%	E	- total energy
%	Gns	- the total number of star generated for galaxy G
%	GSns	- the total number of star generated for galaxy GS
%	Sns	- the total number of star generated for galaxy S
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nt = 2^level+1;
dt = tmax/(nt-1);
tn = [0.0:dt:tmax];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Generate disk stars' position data for all three galaxies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[Gpout Gvout Gns] = addsk(Gm,Gp,Gv,Gn,12,Gr,GR,Gtheta,Gphi);
[GSpout GSvout GSns] = addsk(GSm,GSp,GSv,GSn,12,GSr,GSR,GStheta,GSphi);
[Spout Svout Sns] = addsk(Sm,Sp,Sv,Sn,12,Sr,SR,Stheta,Sphi);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Initalize arrays for efficiency. Total number of elements equals the 
%  number of stars in each galaxy + 3 bulges
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = Gns+GSns+Sns+3;

%  Position, Velocity, Acceelration array
r = zeros(N, 3, nt);
v = zeros(N, 3, nt);
a = zeros(N, 3, nt-1);
%  Potential array
aP = zeros(N, nt);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Adding the positions & velocities elements to arrays.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Add all particles' position coordinates at t = 0 to the matrix
r(1,:,1) = Gp;
r(2,:,1) = GSp;
r(3,:,1) = Sp;
r(4:Gns+3,:,1) = Gpout;
r(Gns+4:Gns+GSns+3,:,1) = GSpout;
r(Gns+GSns+4:N,:,1) = Spout;

%  Add all partcles' velocities to matrix v
v(1,:,1) = Gv;
v(2,:,1) = GSv;
v(3,:,1) = Sv;
v(4:Gns+3,:,1) = Gvout;
v(Gns+4:Gns+GSns+3,:,1) = GSvout;
v(Gns+GSns+4:N,:,1) = Svout;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Add particles' positions at t = 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  accelerations of the bulges
a(1,:,1) = GSm./(sqrt(magnitude(Gp,GSp) + soft^2)).^3.*(GSp-Gp) + Sm./(sqrt(magnitude(Gp,Sp) + soft^2)).^3.*(Sp-Gp);
a(2,:,1) = Gm./(sqrt(magnitude(GSp,Gp) + soft^2)).^3.*(Gp-GSp) + Sm./(sqrt(magnitude(GSp,Sp) + soft^2)).^3.*(Sp-GSp);
a(3,:,1) = Gm./(sqrt(magnitude(Sp,Gp) + soft^2)).^3.*(Gp-Sp) + GSm./(sqrt(magnitude(Sp,GSp) + soft^2)).^3.*(GSp-Sp);

%  find the potential of the bulges
aP(1,1) = -1/2*Gm*GSm./(magsingle(Gp,GSp)+soft)-1/2*Gm*Sm./(magsingle(Gp,Sp)+soft);
aP(2,1) = -1/2*GSm*Gm./(magsingle(GSp,Gp)+soft)-1/2*GSm*Sm./(magsingle(GSp,Sp)+soft);
aP(3,1) = -1/2*Sm*Gm./(magsingle(Sp,Gp)+soft)-1/2*Sm*GSm./(magsingle(Sp,GSp)+soft);

%  get the positions of the bulges
r(1,:,2) = r(1,:,1) + dt*Gv + 1/2*dt^2*(a(1,:,1));
r(2,:,2) = r(2,:,1) + dt*GSv + 1/2*dt^2*(a(2,:,1));
r(3,:,2) = r(3,:,1) + dt*Sv + 1/2*dt^2*(a(3,:,1));


%  acceleration of the stars
%  create matrice for element by element operations
MrG = zeros(N-3,3);
MrG(:,1) = Gp(1);
MrG(:,2) = Gp(2);
MrG(:,3) = Gp(3);
MrGS = zeros(N-3,3);
MrGS(:,1) = GSp(1);
MrGS(:,2) = GSp(2);
MrGS(:,3) = GSp(3);
MrS = zeros(N-3,3);
MrS(:,1) = Sp(1);
MrS(:,2) = Sp(2);
MrS(:,3) = Sp(3);

%  acceleration of stars
a(4:N,:,1) = Gm./(sqrt(magnitude( r(4:N,:,1),MrG )+soft^2)).^3.*( MrG - r(4:N,:,1) ) + GSm./(sqrt(magnitude( r(4:N,:,1),MrGS )+soft^2)).^3.*( MrGS - r(4:N,:,1) ) + Sm./(sqrt(magnitude( r(4:N,:,1),MrS )+soft^2)).^3.*( MrS - r(4:N,:,1) );

%  potential of stars
aP(4:N,1) = -Gm./(magsingle( r(4:N,:,1),MrG )+soft) - GSm./(magsingle( r(4:N,:,1),MrGS )+soft) - Sm./(magsingle( r(4:N,:,1),MrS )+soft);

%  new positions of the stars
r(4:N,:,2) = r(4:N,:,1) + dt*v(4:N,:,1) + 1/2*dt^2*(a(4:N,:,1));


%  compute the velocities of all objects in the system
v(:,:,2) = v(:,:,1) + dt * a(:,:,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Compute the positions of all particles in the system until t max.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for s = 2:nt-1
%  Bulges
a(1,:,s) = GSm./(sqrt(magnitude(r(1,:,s),r(2,:,s)) + soft^2)).^3.*(r(2,:,s)-r(1,:,s)) + Sm./(sqrt(magnitude(r(1,:,s),r(3,:,s)) + soft^2)).^3.*(r(3,:,s)-r(1,:,s)); 
a(2,:,s) = Gm./(sqrt(magnitude(r(2,:,s),r(1,:,s)) + soft^2)).^3.*(r(1,:,s)-r(2,:,s)) + Sm./(sqrt(magnitude(r(2,:,s),r(3,:,s)) + soft^2)).^3.*(r(3,:,s)-r(2,:,s));
a(3,:,s) = Gm./(sqrt(magnitude(r(3,:,s),r(1,:,s)) + soft^2)).^3.*(r(1,:,s)-r(3,:,s)) + GSm./(sqrt(magnitude(r(3,:,s),r(2,:,s)) + soft^2)).^3.*(r(2,:,s)-r(3,:,s));

aP(1,s) = -1/2*Gm*GSm./(magsingle(r(1,:,s),r(2,:,s))+soft)-1/2*Gm*Sm./(magsingle(r(1,:,s),r(3,:,s))+soft);
aP(2,s) = -1/2*GSm*Gm./(magsingle(r(2,:,s),r(1,:,s))+soft)-1/2*GSm*Sm./(magsingle(r(2,:,s),r(3,:,s))+soft);
aP(3,s) = -1/2*Sm*Gm./(magsingle(r(3,:,s),r(1,:,s))+soft)-1/2*Sm*GSm./(magsingle(r(3,:,s),r(2,:,s))+soft);

r(1,:,s+1) = 2*r(1,:,s) - r(1,:,s-1) + dt^2*( a(1,:,s) );
r(2,:,s+1) = 2*r(2,:,s) - r(2,:,s-1) + dt^2*( a(2,:,s) );
r(3,:,s+1) = 2*r(3,:,s) - r(3,:,s-1) + dt^2*( a(3,:,s) );


%  Stars
%  Create matrices for element by element operations
tprG = zeros(N-3,3);
tprG(:,1) = r(1,1,s);
tprG(:,2) = r(1,2,s);
tprG(:,3) = r(1,3,s);
tprGS = zeros(N-3,3);
tprGS(:,1) = r(2,1,s);
tprGS(:,2) = r(2,2,s);
tprGS(:,3) = r(2,3,s);
tprS = zeros(N-3,3);
tprS(:,1) = r(3,1,s);
tprS(:,2) = r(3,2,s);
tprS(:,3) = r(3,3,s);

a(4:N,:,s) = Gm./(sqrt(magnitude( r(4:N,:,s),tprG )+soft^2)).^3.*( tprG-r(4:N,:,s) ) + GSm./(sqrt(magnitude( r(4:N,:,s),tprGS )+soft^2)).^3.*( tprGS-r(4:N,:,s) ) + Sm./(sqrt(magnitude( r(4:N,:,s),tprS )+soft^2)).^3.*( tprS-r(4:N,:,s) );

aP(4:N,s) = -Gm./( magsingle( r(4:N,:,s),tprG )+soft ) - GSm./(magsingle( r(4:N,:,s),tprGS )+soft) - Sm./(magsingle( r(4:N,:,s),tprS )+soft);

r(4:N,:,s+1) = 2*r(4:N,:,s) - r(4:N,:,s-1) + dt^2*( a(4:N,:,s) );


%  Calculate velocities for all particles at all t
v(:,:,s+1) = v(:,:,s) + dt * a(:,:,s);

endfor

%  Calculate potential at t=nt
%  for bulges
aP(1,nt) = -1/2*Gm*GSm./(magsingle(r(1,:,nt),r(2,:,nt))+soft)-1/2*Gm*Sm./(magsingle(r(1,:,nt),r(3,:,nt))+soft);
aP(2,nt) = -1/2*GSm*Gm./(magsingle(r(2,:,nt),r(1,:,nt))+soft)-1/2*GSm*Sm./(magsingle(r(2,:,nt),r(3,:,nt))+soft);
aP(3,nt) = -1/2*Sm*Gm./(magsingle(r(3,:,nt),r(1,:,nt))+soft)-1/2*Sm*GSm./(magsingle(r(3,:,nt),r(2,:,nt))+soft);

%  for stars
tmG = zeros(N-3,3);
tmG(:,1) = r(1,1,nt);
tmG(:,2) = r(1,2,nt);
tmG(:,3) = r(1,3,nt);
tmGS = zeros(N-3,3);
tmGS(:,1) = r(2,1,nt);
tmGS(:,2) = r(2,2,nt);
tmGS(:,3) = r(2,3,nt);
tmS = zeros(N-3,3);
tmS(:,1) = r(3,1,nt);
tmS(:,2) = r(3,2,nt);
tmS(:,3) = r(3,3,nt);
aP(4:N,nt) = -Gm./( magsingle( r(4:N,:,nt),tmG )+soft ) - GSm./(magsingle( r(4:N,:,s),tmGS )+soft) - Sm./(magsingle( r(4:N,:,s),tmS )+soft);


%  generate the total KE matrix
vtot= 0.5*sum(v(:,1,:).^2 + v(:,2,:).^2 + v(:,3,:).^2);
T=zeros(1,nt);
T(1,:) = vtot;
T;

%  generate the total PE matrix
aP;
P = sum(aP);

%  Calculate the total E
E = T + P;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Output data to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

output = out;
if output
  %  Create mass array for the output file
  m=ones(1,N);
  %  mass of G
  m(1,1)=10;
  %  mass of GS
  m(1,2)=10;
  %  mass of S
  m(1,3)=5;
  %  Create colour array
  rgb=zeros(N,3);
  %  Add bulge colors
  rgb(1,:)=[1 1 0];
  rgb(2,:)=[0 1 1];
  rgb(3,:)=[1 0 1];
  %  Add star colors
  cG=zeros(Gns,3);
  cG(:,1)=1;
  cG(:,2)=0;
  cG(:,3)=0;
  rgb(4:Gns+3,:)=cG;
  cGS=zeros(GSns,3);
  cGS(:,1)=0;
  cGS(:,2)=1;
  cGS(:,3)=0;
  rgb(Gns+4:Gns+GSns+3,:)=cGS;
  cS=zeros(Sns,3);
  cS(:,1)=0;
  cS(:,2)=0;
  cS(:,3)=1;
  rgb(Gns+GSns+4:N,:)=cS;
  rout = r ./ 300;
  mout = m ./ 5;
  %  function arguments: nbodyout(fname, t, r, m, rgb)
  if fname == 0
	nbodyout('merger', tn, rout, mout, rgb);
  else
	nbodyout(fname, tn, rout, mout, rgb);
  endif
endif


end







function R = magnitude(r1, r2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Calculate the scalar distance between two objects using their position vectors
%  output it as a vector with the form [mag mag mag] for matrix operations
%
%  Input arguments:
%	r1 - position vector of object 1
%	r2 - position vector of object 2
%
%  Output argument:
%	R  - scalar distance between r1 r2 in vector form for matrix operations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tmp = r2-r1;
% R has the form [mag mag mag]
R = [tmp(:,1).^2+tmp(:,2).^2+tmp(:,3).^2 tmp(:,1).^2+tmp(:,2).^2+tmp(:,3).^2 tmp(:,1).^2+tmp(:,2).^2+tmp(:,3).^2];
end 







function o = magsingle(in1, in2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Calculate the scalar distance between two objects using their position vectors
%  
%
%  Input arguments:
%	r1 - position vector of object 1
%	r2 - position vector of object 2
%
%  Output argument:
%	R - the scalar distance between the two
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tmp = in2-in1;
o = sqrt(tmp(1).^2+tmp(2).^2+tmp(3).^2);
end 
