%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PHYS 210 Term Project Component - Galaxy Disk Generator 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [p v ns] = addsk(mass,gp,gpv,nstars,nvts,rds,Rds,theta,phi)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Input arguments:
%	mass   - the mass of the central bulge
%	gp     - the position of the center of the galaxy
%	gpv    - the velocity of the galaxy
%  	nstars - number of stars
%  	nvts   - number of vertices
%  	rds    - radius of the bulge, within which there are no stars.
%  	Rds    - radius of the disk
%  	theta  - angle between the disk plane and the z-axis
%  	phi    - angle between 
%
%  Output arguments:
% 	p	- matrix of positions
%	v	- matrix of velocities
%	ns	- number of star (excluding stars inside the bulge)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trace = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Calculate positions of disk stars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Find the angle between vertices
dangle = 2*pi/nvts;

%  If we have a point bulge, we keep all generated coordinates
if rds < 0
  %  Create the x y array for efficiency
  x=zeros(1,nstars+nvts);
  y=zeros(1,nstars+nvts);
  %  Create radii array which stores distances to the centre for all stars
  radii=zeros(1,nstars+nvts);
endif

%  Find the coordinates for the vertices
for j = 1:nvts
  v=[Rds*cos(j*dangle), Rds*sin(j*dangle)];
  x(1,j)=v(1,1);
  y(1,j)=v(1,2);
  radii(1,1:nvts)=Rds;
end


%  Generate a random number for x of Vp
Vp_x = Rds*rand(1,1);
%  Since Vp is on a circle with radius 1.25*Rds, it satisfies x^2+y^2=(1.4*Rds)^2. So get y by solving the equation. 
Vp_y = sqrt((Rds*1.25)^2-Vp_x^2);
%  Assign the x and y to point Vp, but Vp is outside the disk, so we do not want it
Vp=[Vp_x Vp_y];

%  Initiate an index for the loop because if the star is within the minimum radius of the disk, we throw it away.
index = 1;

%  Compute positions of stars
for i = 1:nstars 

  %  Choose a vertice randomly
  tmp=fix(1+nvts*rand(1,1));

  %  Find Vq, which is the midpoint of Vp and the chosen vertice
  Vq_x=(x(tmp)+Vp(1,1))/2;
  Vq_y=(y(tmp)+Vp(1,2))/2;

  %  Check if this point is smaller than the minimum radius
  r = radius(Vq_x, Vq_y);
  if rds < r
    x(1,nvts+index)=Vq_x;
    y(1,nvts+index)=Vq_y;
    radii(1,nvts+index)=r;
    index = index+1;
  endif

  %  Assign this new point Vq the active point Vp
  Vp=[Vq_x Vq_y];

end

%  Compute z base on x and theta. Since tan(pi/2) = Inf, so we assign x's to z's
if theta == 0
  z = x;
  x(1,:) = 0;
else
  z = x .* tan(pi/2-theta);
endif

%  let's rotate the coordinates by phi. Using rotation matrix. See Wikipedia rotation matrix
temp_x = x .* cos(phi) - y .* sin(phi);
temp_y = x .* sin(phi) + y .* cos(phi);

%  Assign results to output
x = temp_x; 
y = temp_y;

ns = length(x);

%  Normalize the position vectors and times by corresponding radii.
tmp = [x; y; z]';
pUnit=unitvec(tmp);
radii = [radii; radii; radii]';

%  Create group position matrix(position of the bulge) for element by element operation
pb = zeros(ns,3);
pb(:,1)=gp(1);
pb(:,2)=gp(2);
pb(:,3)=gp(3);
p = radii .* pUnit + pb;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Calculate velocities of disk stars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

v = zeros(ns, 3);

%  Calculate and create disk normal vector matrix
normal = unitvec(cross([x(1) y(1) z(1)],[x(2) y(2) z(2)]));
nor = zeros(ns,3);
nor(:,1)=normal(1);
nor(:,2)=normal(2);
nor(:,3)=normal(3);

%  Calculate the magnitude of velocities
mag = sqrt(mass ./ sqrt(x.^2+y.^2+z.^2));
magnitude = [mag; mag; mag]';

%  Create group velocity matrix for element by element operation
vb = zeros(ns,3);
vb(:,1)=gpv(1);
vb(:,2)=gpv(2);
vb(:,3)=gpv(3);
v=magnitude .* cross(nor,pUnit) + vb;


if trace
  rds
  Rds
  ns
endif


end





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PHYS 210 Term Project Component - Radii Cal
%
%  Input arguments:
%	x - x coordinate of the point in 2D
%	y - y coordinate of the point in 2D
%
%  Output argument
%	d - distance to the center
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function d = radius(x,y)

d = sqrt(x^2+y^2);

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PHYS 210 Term Project Component - univector
%
%  Input arguments:
%	a - vector to be normalized
%	
%  Output argument
%	n - normalized unit vector
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [n] = unitvec(a)
	n = [a(:,1)./sqrt( a(:,1).^2+a(:,2).^2+a(:,3).^2 ) a(:,2)./sqrt( a(:,1).^2+a(:,2).^2+a(:,3).^2 ) a(:,3)./sqrt( a(:,1).^2+a(:,2).^2+a(:,3).^2 )];
end
